package de.fhdw.wtf.parser.test;

import static org.junit.Assert.assertEquals;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.QualifiedName;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;

public class TestName {
	
	@Test
	public void testAddToUnqualifiedName() {
		final IdentifierToken identifierToken = IdentifierToken.create("given", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName givenName = UnqualifiedName.create(identifierToken);
		final IdentifierToken newToken = IdentifierToken.create("toAdd", Position.create("", 0 + 1, 0 + 1, 0));
		final QualifiedName result = (QualifiedName) givenName.addName(newToken);
		
		assertEquals("given", result.getFirst().getIdentifierToken().getIdentifier());
		assertEquals("toAdd", ((UnqualifiedName) result.getRest()).getIdentifierToken().getIdentifier());
	}
	
	@Test
	public void dummyTest() {
		final IdentifierToken identifierToken = IdentifierToken.create("given", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName givenName = UnqualifiedName.create(identifierToken);
		final IdentifierToken newToken = IdentifierToken.create("toAdd", Position.create("", 0 + 1, 0 + 1, 0));
		final Name result = givenName.addName(newToken);
		System.out.println(result);
	}
	
	@Test
	public void testAddToQualifiedNameWithTwoUnqualifiedName() {
		final IdentifierToken identifierTokenFirst =
				IdentifierToken.create("givenfirst", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName givenNameFirst = UnqualifiedName.create(identifierTokenFirst);
		final IdentifierToken identifierTokenRest =
				IdentifierToken.create("givenrest", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName givenNameRest = UnqualifiedName.create(identifierTokenRest);
		final QualifiedName givenName = QualifiedName.create(givenNameFirst, givenNameRest);
		final IdentifierToken newToken = IdentifierToken.create("toAdd", Position.create("", 0 + 1, 0 + 1, 0));
		final QualifiedName result = (QualifiedName) givenName.addName(newToken);
		
		assertEquals("givenfirst", result.getFirst().getIdentifierToken().getIdentifier());
		final QualifiedName rest = (QualifiedName) result.getRest();
		assertEquals("givenrest", rest.getFirst().getIdentifierToken().getIdentifier());
		assertEquals("toAdd", ((UnqualifiedName) rest.getRest()).getIdentifierToken().getIdentifier());
		
	}
	
	@Test
	public void testConcat() {
		final IdentifierToken identifierToken = IdentifierToken.create("group1", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken identifierToken2 = IdentifierToken.create("group2", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken identifierToken3 = IdentifierToken.create("group3", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken identifierToken4 = IdentifierToken.create("class1", Position.create("", 0 + 1, 0 + 1, 0));
		Name name1 = UnqualifiedName.create(identifierToken);
		name1 = name1.addName(identifierToken2);
		Name name2 = UnqualifiedName.create(identifierToken3);
		name2 = name2.addName(identifierToken4);
		Name expResult = UnqualifiedName.create(identifierToken);
		expResult = expResult.addName(identifierToken2);
		expResult = expResult.addName(identifierToken3);
		expResult = expResult.addName(identifierToken4);
		assertEquals(expResult, name1.concat(name2));
	}
	
	@Test
	public void testGetLastAdded() {
		final IdentifierToken identifierToken = IdentifierToken.create("group1", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken identifierToken2 = IdentifierToken.create("group2", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName name1 = UnqualifiedName.create(identifierToken);
		final Name name2 = name1.addName(identifierToken2);
		final Name name3 = UnqualifiedName.create(identifierToken2);
		assertEquals(name1, name1.getLastAddedName());
		assertEquals(name3, name2.getLastAddedName());
	}
	
	@Test
	public void testEqualsPartial() {
		final IdentifierToken identifierToken = IdentifierToken.create("group1", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken identifierToken2 = IdentifierToken.create("group2", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName name1 = UnqualifiedName.create(identifierToken);
		final Name name2 = name1.addName(identifierToken2);
		final Name name3 = UnqualifiedName.create(identifierToken2);
		final Name name4 = name3.addName(identifierToken);
		assertEquals(true, name2.equalsPartial(name4));
	}
}
